<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM Api access manager
 * 
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_PlusPackage_Api_Manager {

    /**
     * Instance of itself
     * 
     * @var AAM_PlusPackage_Api_Manager
     * 
     * @access private 
     */
    private static $_instance = null;
    
    /**
     * Map of routes and resources
     * 
     * @var array
     * 
     * @access protected 
     */
    protected $resources = array(
        'term' => array (
            '/wp/v2/categories',
            '/wp/v2/categories/(?P<id>[\d]+)'
        ),
        'comment' => array (
            '/wp/v2/comments',
            '/wp/v2/comments/(?P<id>[\d]+)'
        )
    );
    
    /**
     * Construct the manager
     * 
     * @return void
     * 
     * @access public
     */
    protected function __construct() {
        // Register any additional endpoints with ConfigPress
        $additional = AAM::api()->getConfig('rest.manage.endpoint');
        
        if (!empty($additional) && is_array($additional)) {
            $this->resources = array_merge_recursive($this->resources, $additional);
        }
    }
    
    /**
     * Authorize RESTful action before it is dispatched by RESTful Server
     * 
     * @param mixed  $response
     * @param object $handler
     * @param object $request
     * 
     * @return mixed
     * 
     * @access public
     */
    public static function dispatch($response, $handler, $request) {
        $result = null;
        
        if (is_null(self::$_instance)) {
            self::$_instance = new self;
        }
        
        foreach(self::$_instance->resources as $res => $routes) {
            foreach($routes as $regex) {
                // Route to work with single post
                if(preg_match('#^' . $regex . '$#i', $request->get_route())) {
                    $classname = 'AAM_PlusPackage_Api_Rest_Resource_' . ucfirst($res);
                    $result    = $classname::getInstance()->authorize($request);
                }
            }
        }
        
        return (is_null($result) ? $response : $result);
    }
    
}