<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM RESTful Comment Resource
 * 
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_PlusPackage_Api_Rest_Resource_Comment {
    
    /**
     * Instance of itself
     * 
     * @var AAM_PlusPackage_Api_Rest_Resource_Comment
     * 
     * @access private 
     */
    private static $_instance = null;
    
    /**
     * Authorize Post actions
     * 
     * @param WP_REST_Request $request
     * 
     * @return WP_Error|null
     * 
     * @access public
     */
    public function authorize($request) {
        $response = null;
        
        if ($request['id']) {
            switch($request->get_method()) {
                case 'POST':
                case 'PUT':
                case 'PATCH':
                    $response = $this->checkUpdateComment();
                    break;

                case 'DELETE':
                    $response = $this->checkDeleteComment($request['force']);
                    break;

                default:
                    break;
            }
        }
        
        return $response;
    }
    
    /**
     * 
     * @return type
     */
    protected function checkUpdateComment() {
        $result = null;
        
        if (!AAM::getUser()->hasCapability('edit_comment')) {
            $result = new WP_Error(
                'rest_comment_edit_denied', 
                "You are not allowed to edit comments. Access denied",
                array(
                    'status' => 403
                )
            );
        }
        
        return $result;
    }
    
    /**
     * 
     * @return type
     */
    protected function checkDeleteComment($force) {
        $result = null;
        
        if ($force) {
            if (!AAM::getUser()->hasCapability('delete_comment')) {
                $result = new WP_Error(
                    'rest_comment_delete_denied', 
                    "You are not allowed to delete this comment. Access denied",
                    array('status' => 403)
                );
            }
        } elseif (!AAM::getUser()->hasCapability('trash_comment')) {
            $result = new WP_Error(
                'rest_comment_trash_denied', 
                "You are not allowed to trash this comment. Access denied",
                array('status' => 403)
            );
        }
        
        return $result;
    }
    
    /**
     * Alias for the bootstrap
     * 
     * @return AAM_PlusPackage_Api_Rest_Resource_Term
     * 
     * @access public
     * @static
     */
    public static function getInstance() {
        return self::bootstrap();
    }
    
    /**
     * Bootstrap authorization layer
     * 
     * @return AAM_PlusPackage_Api_Rest_Resource_Term
     * 
     * @access public
     */
    public static function bootstrap() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self;
        }
        
        return self::$_instance;
    }
}