<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * Taxonomy object
 * 
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_Object_Taxonomy extends AAM_Core_Object {

    /**
     * Taxonomy
     * 
     * @var string
     * 
     * @access private
     * @todo   When min required WP version will be 4.7.0, migrate it to WP_Taxonomy
     */
    private $_taxonomy = null;
    
    /**
     * Post Type taxonomy attached to
     * 
     * @var string
     * 
     * @access private 
     */
    private $_postType = null;
    
    /**
     * Object Id
     * 
     * @var string
     * 
     * @access private 
     */
    private $_id = null;

    /**
     * Constructor
     *
     * @param AAM_Core_Subject $subject
     * @param int              $taxonomy
     * @param string           $postType
     *
     * @return void
     *
     * @access public
     */
    public function __construct(AAM_Core_Subject $subject, $id) {
        parent::__construct($subject);

        $this->_id = $id;
        list($this->_postType, $this->_taxonomy) = explode('|', $id);
        
        //make sure that we are dealing with term object
        if ($this->_id) {
            $this->initialize();
        }
    }
    
    /**
     * 
     */
    public function initialize() {
        $this->read();
    }
    
    /**
     * Get Object ID
     * 
     * @return string
     * 
     * @access public
     */
    public function getId() {
        return $this->_id;
    }
    
    /**
     * Get taxonomy name
     * 
     * @return string
     * 
     * @access public
     */
    public function getTaxonomy() {
        return $this->_taxonomy;
    }

    /**
     * Read the term
     * 
     * @return void
     * 
     * @access protected
     */
    protected function read() {
        $subject = $this->getSubject();
        
        $option = $subject->readOption('taxonomy', $this->_id);
        $this->setOverwritten(!empty($option));

        if (empty($option)) {
            $default = AAM::api()->getPolicyManager($subject)->getParam(
                "{$this->_postType}:default:{$this->_taxonomy}"
            );
            if (!empty($default)) {
                $id = $default;
                if (!is_numeric($default)) {
                    $term = get_term_by('slug', $default, $this->_taxonomy);
                    $id = (is_a($term, 'WP_Term') ? $term->term_id : 0);
                }
                if ($id) {
                    $option = array('default' => $id);
                }
            }
        }

        $this->setOption($option);
    }
    
    /**
     * Save options
     * 
     * @param string  $param
     * @param boolean $value
     * 
     * @return boolean
     * 
     * @access public
     */
    public function save($param, $value) {
        $option         = $this->getOption();
        $option[$param] = $value;
        
        $result = $this->getSubject()->updateOption($option, 'taxonomy', $this->_id);
        
        if ($result) {
            $this->setOption($option);
        }
        
        return $result;
    }
    
    /**
     * 
     * @return type
     */
    public function reset() {
        return $this->getSubject()->deleteOption('taxonomy', $this->_id);
    }

    /**
     * Check if property is selected
     * 
     * @param string $param
     * 
     * @return boolean
     * 
     * @access public
     */
    public function has($param) {
        $option = $this->getOption();
        
        return !empty($option[$param]);
    }
    
    /**
     * 
     * @param type $group
     * @param type $action
     * @return type
     */
    public function get($param) {
        $option = $this->getOption();
        
        return isset($option[$param]) ? $option[$param] : null;
    }
    
    /**
     * 
     * @param type $external
     * @return type
     */
    public function mergeOption($external) {
        return AAM::api()->mergeSettings($external, $this->getOption(), 'taxonomy');
    }
    
}